@@ -94,7 +94,7 @@ func (sm SymbolMap) symbolicate(m *macho.File, name string, sigs Symbolicator, q
 		return fmt.Errorf("failed to get data from __TEXT_EXEC.__text section: %v", err)
 	}
 
-	engine := disass.NewMachoDisass(m, &map[uint64]string{}, &disass.Config{
+	engine := disass.NewMachoDisass(m, &disass.Config{
 		Data:         data,
 		StartAddress: text.Addr,
 		Quite:        true,
