@@ -28,7 +28,14 @@ const config = {
 
   onBrokenLinks: "throw",
   onBrokenMarkdownLinks: "warn",
-
+  future: {
+    v4: {
+      removeLegacyPostBuildHeadAttribute: true, // required
+    },
+    experimental_faster: {
+      ssgWorkerThreads: true,
+    },
+  },
   // Even if you don't use internalization, you can use this field to set useful
   // metadata like html lang. For example, if your site is Chinese, you may want
   // to replace "en" with "zh-Hans".
@@ -103,7 +110,7 @@ const config = {
       },
     ],
   ],
-  // plugins: [require.resolve("@cmfcmf/docusaurus-search-local")],
+
   themeConfig:
     /** @type {import('@docusaurus/preset-classic').ThemeConfig} */
     ({
@@ -164,6 +171,7 @@ const config = {
             sidebarId: "api",
           },
           { to: '/blog', label: 'Blog', position: 'left' },
+          { to: '/entitlements', label: 'Entitlements', position: 'left' },
           // {
           //   label: "DIFF",
           //   to: "/diff",
